+
A simple tui to view & control docker containers
@@ -17,7 +17,7 @@
Images built for `linux/amd64`, `linux/arm64`, and `linux/arm/v6`
-`docker run --rm -it --volume /var/run/docker.sock:/var/run/docker.sock:ro --pull=always mrjackwills/oxker`
+`docker run --rm -it -v /var/run/docker.sock:/var/run/docker.sock:ro --pull=always mrjackwills/oxker`
## Help
diff --git a/src/app_data/container_state.rs b/src/app_data/container_state.rs
index 03c0c1c..b8b32b9 100644
--- a/src/app_data/container_state.rs
+++ b/src/app_data/container_state.rs
@@ -7,11 +7,49 @@ use tui::{
use super::Header;
-
const ONE_KB: f64 = 1000.0;
const ONE_MB: f64 = ONE_KB * 1000.0;
const ONE_GB: f64 = ONE_MB * 1000.0;
+#[derive(Debug, Clone, Eq, Hash, PartialEq)]
+pub struct ContainerId(String);
+
+impl From
for ContainerId {
+ fn from(x: String) -> Self {
+ Self(x)
+ }
+}
+
+impl From<&String> for ContainerId {
+ fn from(x: &String) -> Self {
+ Self(x.clone())
+ }
+}
+
+impl From<&str> for ContainerId {
+ fn from(x: &str) -> Self {
+ Self(x.to_owned())
+ }
+}
+
+impl ContainerId {
+ pub fn get(&self) -> &str {
+ self.0.as_str()
+ }
+}
+
+impl Ord for ContainerId {
+ fn cmp(&self, other: &Self) -> Ordering {
+ self.0.cmp(&other.0)
+ }
+}
+
+impl PartialOrd for ContainerId {
+ fn partial_cmp(&self, other: &Self) -> Option {
+ Some(self.cmp(other))
+ }
+}
+
#[derive(Debug, Clone)]
pub struct StatefulList {
pub state: ListState,
@@ -25,6 +63,7 @@ impl StatefulList {
items,
}
}
+
pub fn end(&mut self) {
let len = self.items.len();
if len > 0 {
@@ -70,7 +109,7 @@ impl StatefulList {
pub fn get_state_title(&self) -> String {
if self.items.is_empty() {
- String::from("")
+ String::new()
} else {
let len = self.items.len();
let c = self
@@ -83,7 +122,7 @@ impl StatefulList {
}
/// States of the container
-#[derive(Clone, Debug, PartialEq, Eq, PartialOrd)]
+#[derive(Debug, Clone, Copy, Eq, PartialEq, PartialOrd)]
pub enum State {
Dead,
Exited,
@@ -95,25 +134,25 @@ pub enum State {
}
impl State {
- pub const fn get_color(&self) -> Color {
+ pub const fn get_color(self) -> Color {
match self {
- Self::Running => Color::Green,
+ Self::Paused => Color::Yellow,
Self::Removing => Color::LightRed,
Self::Restarting => Color::LightGreen,
- Self::Paused => Color::Yellow,
+ Self::Running => Color::Green,
_ => Color::Red,
}
}
// Dirty way to create order for the state, rather than impl Ord
- pub const fn order(&self) -> &'static str {
+ pub const fn order(self) -> u8 {
match self {
- Self::Running => "a",
- Self::Paused => "b",
- Self::Restarting => "c",
- Self::Removing => "d",
- Self::Exited => "e",
- Self::Dead => "f",
- Self::Unknown => "g",
+ Self::Running => 0,
+ Self::Paused => 1,
+ Self::Restarting => 2,
+ Self::Removing => 3,
+ Self::Exited => 4,
+ Self::Dead => 5,
+ Self::Unknown => 6,
}
}
}
@@ -162,27 +201,28 @@ impl fmt::Display for State {
}
/// Items for the container control list
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, Copy)]
pub enum DockerControls {
Pause,
- Unpause,
Restart,
- Stop,
Start,
+ Stop,
+ Unpause,
}
impl DockerControls {
- pub const fn get_color(&self) -> Color {
+ pub const fn get_color(self) -> Color {
match self {
+ Self::Pause => Color::Yellow,
+ Self::Restart => Color::Magenta,
Self::Start => Color::Green,
Self::Stop => Color::Red,
- Self::Restart => Color::Magenta,
- Self::Pause => Color::Yellow,
Self::Unpause => Color::Blue,
}
}
- pub fn gen_vec(state: &State) -> Vec {
+ /// Docker commands available depending on the containers state
+ pub fn gen_vec(state: State) -> Vec {
match state {
State::Dead | State::Exited => vec![Self::Start, Self::Restart],
State::Paused => vec![Self::Unpause, Self::Stop],
@@ -197,10 +237,10 @@ impl fmt::Display for DockerControls {
fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
let disp = match self {
Self::Pause => "pause",
- Self::Unpause => "unpause",
Self::Restart => "restart",
- Self::Stop => "stop",
Self::Start => "start",
+ Self::Stop => "stop",
+ Self::Unpause => "unpause",
};
write!(f, "{}", disp)
}
@@ -213,7 +253,7 @@ pub trait Stats {
/// Struct for frequently updated CPU stats
/// So can use custom display formatter
/// Use trait Stats for use as generic in draw_chart function
-#[derive(Clone, Debug)]
+#[derive(Debug, Default, Clone, Copy)]
pub struct CpuStats {
value: f64,
}
@@ -266,7 +306,7 @@ impl fmt::Display for CpuStats {
/// Struct for frequently updated memory usage stats
/// So can use custom display formatter
/// Use trait Stats for use as generic in draw_chart function
-#[derive(Clone, Debug, Eq)]
+#[derive(Debug, Default, Clone, Copy, Eq)]
pub struct ByteStats {
value: u64,
}
@@ -297,17 +337,18 @@ impl ByteStats {
self.value = value;
}
}
+
+#[allow(clippy::cast_precision_loss)]
impl Stats for ByteStats {
fn get_value(&self) -> f64 {
self.value as f64
}
}
-// convert from bytes to kB, MB, GB etc
+/// convert from bytes to kB, MB, GB etc
impl fmt::Display for ByteStats {
fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
- // TODO these can be consts outside of this definition
- let as_f64 = self.value as f64;
+ let as_f64 = self.get_value();
let p = match as_f64 {
x if x >= ONE_GB => format!("{y:.2} GB", y = as_f64 / ONE_GB),
x if x >= ONE_MB => format!("{y:.2} MB", y = as_f64 / ONE_MB),
@@ -318,12 +359,15 @@ impl fmt::Display for ByteStats {
}
}
+pub type MemTuple = (Vec<(f64, f64)>, ByteStats, State);
+pub type CpuTuple = (Vec<(f64, f64)>, CpuStats, State);
+
/// Info for each container
#[derive(Debug, Clone)]
pub struct ContainerItem {
pub cpu_stats: VecDeque,
pub docker_controls: StatefulList,
- pub id: String,
+ pub id: ContainerId,
pub image: String,
pub last_updated: u64,
pub logs: StatefulList>,
@@ -336,57 +380,58 @@ pub struct ContainerItem {
pub status: String,
}
-pub type MemTuple = (Vec<(f64, f64)>, ByteStats, State);
-pub type CpuTuple = (Vec<(f64, f64)>, CpuStats, State);
-
impl ContainerItem {
/// Create a new container item
- pub fn new(id: String, status: String, image: String, state: State, name: String) -> Self {
- let mut docker_controls = StatefulList::new(DockerControls::gen_vec(&state));
+ pub fn new(id: ContainerId, status: String, image: String, state: State, name: String) -> Self {
+ let mut docker_controls = StatefulList::new(DockerControls::gen_vec(state));
docker_controls.start();
+ let mut logs = StatefulList::new(vec![]);
+ logs.end();
Self {
cpu_stats: VecDeque::with_capacity(60),
docker_controls,
id,
image,
last_updated: 0,
- logs: StatefulList::new(vec![]),
- mem_limit: ByteStats::new(0),
+ logs,
+ mem_limit: ByteStats::default(),
mem_stats: VecDeque::with_capacity(60),
name,
- rx: ByteStats::new(0),
- tx: ByteStats::new(0),
+ rx: ByteStats::default(),
+ tx: ByteStats::default(),
state,
status,
}
}
- /// Find the max value in the last 30 items in the cpu stats vec
+ /// Find the max value in the cpu stats VecDeque
fn max_cpu_stats(&self) -> CpuStats {
match self.cpu_stats.iter().max() {
- Some(value) => value.clone(),
- None => CpuStats::new(0.0),
+ Some(value) => *value,
+ None => CpuStats::default(),
}
}
- /// Find the max value in the last 30 items in the mem stats vec
+ /// Find the max value in the mem stats VecDeque
fn max_mem_stats(&self) -> ByteStats {
match self.mem_stats.iter().max() {
- Some(value) => value.clone(),
- None => ByteStats::new(0),
+ Some(value) => *value,
+ None => ByteStats::default(),
}
}
/// Convert cpu stats into a vec for the charts function
+ #[allow(clippy::cast_precision_loss)]
fn get_cpu_dataset(&self) -> Vec<(f64, f64)> {
self.cpu_stats
.iter()
.enumerate()
- .map(|i| (i.0 as f64, i.1.value))
+ .map(|i| (i.0 as f64, i.1.value as f64))
.collect::>()
}
- /// Convert mem stats into a vec for the charts function
+ /// Convert mem stats into a Vec for the charts function
+ #[allow(clippy::cast_precision_loss)]
fn get_mem_dataset(&self) -> Vec<(f64, f64)> {
self.mem_stats
.iter()
@@ -397,20 +442,12 @@ impl ContainerItem {
/// Get all cpu chart data
fn get_cpu_chart_data(&self) -> CpuTuple {
- (
- self.get_cpu_dataset(),
- self.max_cpu_stats(),
- self.state.clone(),
- )
+ (self.get_cpu_dataset(), self.max_cpu_stats(), self.state)
}
/// Get all mem chart data
fn get_mem_chart_data(&self) -> MemTuple {
- (
- self.get_mem_dataset(),
- self.max_mem_stats(),
- self.state.clone(),
- )
+ (self.get_mem_dataset(), self.max_mem_stats(), self.state)
}
/// Get chart info for cpu & memory in one function
@@ -421,7 +458,7 @@ impl ContainerItem {
}
/// Container information panel headings + widths, for nice pretty formatting
-#[derive(Debug)]
+#[derive(Debug, Clone, Copy)]
pub struct Columns {
pub state: (Header, usize),
pub status: (Header, usize),
diff --git a/src/app_data/mod.rs b/src/app_data/mod.rs
index 6c7924d..abb6f8b 100644
--- a/src/app_data/mod.rs
+++ b/src/app_data/mod.rs
@@ -9,7 +9,7 @@ use crate::{app_error::AppError, parse_args::CliArgs, ui::log_sanitizer};
pub use container_state::*;
/// Global app_state, stored in an Arc
-#[derive(Debug)]
+#[derive(Debug, Clone)]
pub struct AppData {
args: CliArgs,
error: Option,
@@ -20,13 +20,13 @@ pub struct AppData {
sorted_by: Option<(Header, SortedOrder)>,
}
-#[derive(Debug, Clone, PartialEq, Eq)]
+#[derive(Debug, Clone, Copy, Eq, PartialEq)]
pub enum SortedOrder {
Asc,
Desc,
}
-#[derive(Debug, Clone, PartialEq, Hash, Eq)]
+#[derive(Debug, Clone, Copy, Eq, Hash, PartialEq)]
pub enum Header {
State,
Status,
@@ -58,21 +58,20 @@ impl fmt::Display for Header {
}
impl AppData {
- pub fn get_sorted(&self) -> Option<(Header, SortedOrder)> {
- self.sorted_by.clone()
+ pub const fn get_sorted(&self) -> Option<(Header, SortedOrder)> {
+ self.sorted_by
}
/// Change the sorted order, also set the selected container state to match new order
pub fn set_sorted(&mut self, x: Option<(Header, SortedOrder)>) {
self.sorted_by = x;
- let id = self.get_selected_container_id();
self.sort_containers();
- self.containers.state.select(
- self.containers
- .items
- .iter()
- .position(|i| Some(i.id.clone()) == id),
- );
+ self.containers
+ .state
+ .select(self.containers.items.iter().position(|i| {
+ self.get_selected_container_id()
+ .map_or(false, |id| i.id == id)
+ }));
}
/// Generate a default app_state
pub fn default(args: CliArgs) -> Self {
@@ -106,8 +105,7 @@ impl AppData {
.state
.selected()
{
- output =
- Some(self.containers.items[index].docker_controls.items[control_index].clone());
+ output = Some(self.containers.items[index].docker_controls.items[control_index]);
}
}
output
@@ -116,34 +114,42 @@ impl AppData {
/// Change selected choice of docker commands of selected container
pub fn docker_command_next(&mut self) {
if let Some(index) = self.containers.state.selected() {
- self.containers.items[index].docker_controls.next();
+ if let Some(i) = self.containers.items.get_mut(index) {
+ i.docker_controls.next();
+ }
}
}
/// Change selected choice of docker commands of selected container
pub fn docker_command_previous(&mut self) {
if let Some(index) = self.containers.state.selected() {
- self.containers.items[index].docker_controls.previous();
+ if let Some(i) = self.containers.items.get_mut(index) {
+ i.docker_controls.previous();
+ }
}
}
/// Change selected choice of docker commands of selected container
pub fn docker_command_start(&mut self) {
if let Some(index) = self.containers.state.selected() {
- self.containers.items[index].docker_controls.start();
+ if let Some(i) = self.containers.items.get_mut(index) {
+ i.docker_controls.start();
+ }
}
}
/// Change selected choice of docker commands of selected container
pub fn docker_command_end(&mut self) {
if let Some(index) = self.containers.state.selected() {
- self.containers.items[index].docker_controls.end();
+ if let Some(i) = self.containers.items.get_mut(index) {
+ i.docker_controls.end();
+ }
}
}
/// return single app_state error
- pub fn get_error(&self) -> Option {
- self.error.clone()
+ pub const fn get_error(&self) -> Option {
+ self.error
}
/// remove single app_state error
@@ -157,39 +163,33 @@ impl AppData {
}
/// Find the id of the currently selected container.
- /// If any containers on system, will always return a string.
+ /// If any containers on system, will always return a ContainerId
/// Only returns None when no containers found.
- pub fn get_selected_container_id(&self) -> Option {
+ pub fn get_selected_container_id(&self) -> Option {
let mut output = None;
if let Some(index) = self.containers.state.selected() {
- let id = self
- .containers
- .items
- .iter()
- .skip(index)
- .take(1)
- .map(|i| i.id.clone())
- .collect::();
- output = Some(id);
+ if let Some(x) = self.containers.items.get(index) {
+ output = Some(x.id.clone());
+ }
}
output
}
/// Sort the containers vec, based on a heading, either ascending or descending
pub fn sort_containers(&mut self) {
- if let Some((head, so)) = self.sorted_by.as_ref() {
+ if let Some((head, ord)) = self.sorted_by.as_ref() {
match head {
- Header::State => match so {
+ Header::State => match ord {
SortedOrder::Desc => self
.containers
.items
- .sort_by(|a, b| a.state.order().cmp(b.state.order())),
+ .sort_by(|a, b| a.state.order().cmp(&b.state.order())),
SortedOrder::Asc => self
.containers
.items
- .sort_by(|a, b| b.state.order().cmp(a.state.order())),
+ .sort_by(|a, b| b.state.order().cmp(&a.state.order())),
},
- Header::Status => match so {
+ Header::Status => match ord {
SortedOrder::Asc => self
.containers
.items
@@ -199,7 +199,7 @@ impl AppData {
.items
.sort_by(|a, b| b.status.cmp(&a.status)),
},
- Header::Cpu => match so {
+ Header::Cpu => match ord {
SortedOrder::Asc => self
.containers
.items
@@ -209,7 +209,7 @@ impl AppData {
.items
.sort_by(|a, b| b.cpu_stats.back().cmp(&a.cpu_stats.back())),
},
- Header::Memory => match so {
+ Header::Memory => match ord {
SortedOrder::Asc => self
.containers
.items
@@ -219,25 +219,25 @@ impl AppData {
.items
.sort_by(|a, b| b.mem_stats.back().cmp(&a.mem_stats.back())),
},
- Header::Id => match so {
+ Header::Id => match ord {
SortedOrder::Asc => self.containers.items.sort_by(|a, b| a.id.cmp(&b.id)),
SortedOrder::Desc => self.containers.items.sort_by(|a, b| b.id.cmp(&a.id)),
},
- Header::Image => match so {
+ Header::Image => match ord {
SortedOrder::Asc => self.containers.items.sort_by(|a, b| a.image.cmp(&b.image)),
SortedOrder::Desc => {
self.containers.items.sort_by(|a, b| b.image.cmp(&a.image));
}
},
- Header::Name => match so {
+ Header::Name => match ord {
SortedOrder::Asc => self.containers.items.sort_by(|a, b| a.name.cmp(&b.name)),
SortedOrder::Desc => self.containers.items.sort_by(|a, b| b.name.cmp(&a.name)),
},
- Header::Rx => match so {
+ Header::Rx => match ord {
SortedOrder::Asc => self.containers.items.sort_by(|a, b| a.rx.cmp(&b.rx)),
SortedOrder::Desc => self.containers.items.sort_by(|a, b| b.rx.cmp(&a.rx)),
},
- Header::Tx => match so {
+ Header::Tx => match ord {
SortedOrder::Asc => self.containers.items.sort_by(|a, b| a.tx.cmp(&b.tx)),
SortedOrder::Desc => self.containers.items.sort_by(|a, b| b.tx.cmp(&a.tx)),
},
@@ -259,41 +259,50 @@ impl AppData {
/// Get the title for log panel for selected container
/// will be "logs x/x"
pub fn get_log_title(&self) -> String {
- self.get_selected_log_index().map_or_else(
- || String::from(""),
- |index| self.containers.items[index].logs.get_state_title(),
- )
+ self.get_selected_log_index()
+ .map_or("".to_owned(), |index| {
+ self.containers.items[index].logs.get_state_title()
+ })
}
/// select next selected log line
pub fn log_next(&mut self) {
if let Some(index) = self.get_selected_log_index() {
- self.containers.items[index].logs.next();
+ if let Some(i) = self.containers.items.get_mut(index) {
+ i.logs.next();
+ }
}
}
/// select previous selected log line
pub fn log_previous(&mut self) {
if let Some(index) = self.get_selected_log_index() {
- self.containers.items[index].logs.previous();
+ if let Some(i) = self.containers.items.get_mut(index) {
+ i.logs.previous();
+ }
}
}
/// select last selected log line
pub fn log_end(&mut self) {
if let Some(index) = self.get_selected_log_index() {
- self.containers.items[index].logs.end();
+ if let Some(i) = self.containers.items.get_mut(index) {
+ i.logs.end();
+ }
}
}
/// select first selected log line
pub fn log_start(&mut self) {
if let Some(index) = self.get_selected_log_index() {
- self.containers.items[index].logs.start();
+ if let Some(i) = self.containers.items.get_mut(index) {
+ i.logs.start();
+ }
}
}
- pub fn initialised(&mut self, all_ids: &[(bool, String)]) -> bool {
+ /// Check if the initial parsing has been completed, by making sure that all ids given (which are running) have a non empty cpu_stats vecdec
+ pub fn initialised(&mut self, all_ids: &[(bool, ContainerId)]) -> bool {
let count_is_running = all_ids.iter().filter(|i| i.0).count();
let number_with_cpu_status = self
.containers
@@ -315,17 +324,18 @@ impl AppData {
let mut output = Columns::new();
let count = |x: &String| x.chars().count();
+ // Should probably find a refactor here somewhere
for container in &self.containers.items {
let cpu_count = count(
&container
.cpu_stats
.back()
- .unwrap_or(&CpuStats::new(0.0))
+ .unwrap_or(&CpuStats::default())
.to_string(),
);
let mem_count = count(&format!(
"{} / {}",
- container.mem_stats.back().unwrap_or(&ByteStats::new(0)),
+ container.mem_stats.back().unwrap_or(&ByteStats::default()),
container.mem_limit
));
@@ -365,7 +375,7 @@ impl AppData {
}
/// Get all containers ids
- pub fn get_all_ids(&self) -> Vec {
+ pub fn get_all_ids(&self) -> Vec {
self.containers
.items
.iter()
@@ -373,15 +383,15 @@ impl AppData {
.collect::>()
}
- /// find container given id
- fn get_container_by_id(&mut self, id: &str) -> Option<&mut ContainerItem> {
- self.containers.items.iter_mut().find(|i| i.id == id)
+ /// return a mutable container by given id
+ fn get_container_by_id(&mut self, id: &ContainerId) -> Option<&mut ContainerItem> {
+ self.containers.items.iter_mut().find(|i| &i.id == id)
}
/// Update container mem, cpu, & network stats, in single function so only need to call .lock() once
pub fn update_stats(
&mut self,
- id: &str,
+ id: &ContainerId,
cpu_stat: Option,
mem_stat: Option,
mem_limit: u64,
@@ -410,18 +420,18 @@ impl AppData {
}
/// Update, or insert, containers
- pub fn update_containers(&mut self, containers: &mut [ContainerSummary]) {
+ pub fn update_containers(&mut self, all_containers: &mut [ContainerSummary]) {
let all_ids = self.get_all_ids();
- if !containers.is_empty() && self.containers.state.selected().is_none() {
+ if !all_containers.is_empty() && self.containers.state.selected().is_none() {
self.containers.start();
}
for (index, id) in all_ids.iter().enumerate() {
- if !containers
+ if !all_containers
.iter()
.filter_map(|i| i.id.as_ref())
- .any(|x| x == id)
+ .any(|x| x == id.get())
{
// If removed container is currently selected, then change selected to previous
// This will default to 0 in any edge cases
@@ -434,63 +444,54 @@ impl AppData {
}
}
}
+ // Trim a &String and return String
+ let trim_owned = |x: &String| x.trim().to_owned();
- for i in containers.iter_mut() {
+ for i in all_containers {
if let Some(id) = i.id.as_ref() {
- // maybe if no name then continue?
- let name = i.names.as_mut().map_or("".to_owned(), |n| {
- n.get_mut(0).map_or("".to_owned(), |f| {
+ let name = i.names.as_mut().map_or("".to_owned(), |names| {
+ names.first_mut().map_or("".to_owned(), |f| {
if f.starts_with('/') {
f.remove(0);
}
- f.clone()
+ (*f).to_string()
})
});
- let state = State::from(
- i.state
- .as_ref()
- .map_or("dead".to_owned(), |f| f.trim().to_owned()),
- );
- let status = i
- .status
- .as_ref()
- .map_or("".to_owned(), |f| f.trim().to_owned());
+ let state = State::from(i.state.as_ref().map_or("dead".to_owned(), trim_owned));
+ let status = i.status.as_ref().map_or("".to_owned(), trim_owned);
let image = i
.image
.as_ref()
.map_or("".to_owned(), std::clone::Clone::clone);
- if let Some(current_container) = self.get_container_by_id(id) {
- if current_container.name != name {
- current_container.name = name;
+ let id = ContainerId::from(id);
+ // If container info already in containers Vec, then just update details
+ if let Some(item) = self.get_container_by_id(&id) {
+ if item.name != name {
+ item.name = name;
};
- if current_container.status != status {
- current_container.status = status;
+ if item.status != status {
+ item.status = status;
};
- if current_container.state != state {
- current_container.docker_controls.items = DockerControls::gen_vec(&state);
-
+ if item.state != state {
+ item.docker_controls.items = DockerControls::gen_vec(state);
// Update the list state, needs to be None if the gen_vec returns an empty vec
match state {
State::Removing | State::Restarting | State::Unknown => {
- current_container.docker_controls.state.select(None);
+ item.docker_controls.state.select(None);
}
- _ => current_container.docker_controls.start(),
+ _ => item.docker_controls.start(),
};
- current_container.state = state;
+ item.state = state;
};
- if current_container.image != image {
- // limit image name to 64 chars?
- // current_container.image = image.chars().into_iter().take(64).collect();
- current_container.image = image;
+ if item.image != image {
+ item.image = image;
};
+ // else container not known, so make new ContainerItem and push into containers Vec
} else {
- // limit image name to 64 chars?
- // let mut container = ContainerItem::new(id.clone(), status, image.chars().into_iter().take(64).collect(), state, name);
- let mut container = ContainerItem::new(id.clone(), status, image, state, name);
- container.logs.end();
+ let container = ContainerItem::new(id, status, image, state, name);
self.containers.items.push(container);
}
}
@@ -498,7 +499,7 @@ impl AppData {
}
/// update logs of a given container, based on id
- pub fn update_log_by_id(&mut self, output: &[String], id: &str) {
+ pub fn update_log_by_id(&mut self, output: &[String], id: &ContainerId) {
let tz = Self::get_systemtime();
let color = self.args.color;
let raw = self.args.raw;
@@ -507,17 +508,19 @@ impl AppData {
container.last_updated = tz;
let current_len = container.logs.items.len();
- for i in output.iter() {
+ for i in output {
let lines = if color {
log_sanitizer::colorize_logs(i)
} else if raw {
- log_sanitizer::raw(i.clone())
+ log_sanitizer::raw(i)
} else {
log_sanitizer::remove_ansi(i)
};
container.logs.items.push(ListItem::new(lines));
}
+ // Set the logs selected row for each container
+ // Either when no long currently selected, or currently selected (before updated) is already at end
if container.logs.state.selected().is_none()
|| container.logs.state.selected().map_or(1, |f| f + 1) == current_len
{
diff --git a/src/app_error.rs b/src/app_error.rs
index b1bcd83..1f90db6 100644
--- a/src/app_error.rs
+++ b/src/app_error.rs
@@ -3,12 +3,12 @@ use std::fmt;
/// app errors to set in global state
#[allow(unused)]
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, Copy)]
pub enum AppError {
+ DockerCommand(DockerControls),
DockerConnect,
DockerInterval,
InputPoll,
- DockerCommand(DockerControls),
MouseCapture(bool),
Terminal,
}
@@ -17,15 +17,15 @@ pub enum AppError {
impl fmt::Display for AppError {
fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
match self {
+ Self::DockerCommand(s) => write!(f, "Unable to {} container", s),
Self::DockerConnect => write!(f, "Unable to access docker daemon"),
Self::DockerInterval => write!(f, "Docker update interval needs to be greater than 0"),
Self::InputPoll => write!(f, "Unable to poll user input"),
- Self::Terminal => write!(f, "Unable to draw to terminal"),
- Self::DockerCommand(s) => write!(f, "Unable to {} container", s),
Self::MouseCapture(x) => {
let reason = if *x { "en" } else { "dis" };
write!(f, "Unbale to {}able mouse capture", reason)
}
+ Self::Terminal => write!(f, "Unable to draw to terminal"),
}
}
}
diff --git a/src/docker_data/message.rs b/src/docker_data/message.rs
index a730830..f7de6f5 100644
--- a/src/docker_data/message.rs
+++ b/src/docker_data/message.rs
@@ -1,10 +1,12 @@
+use crate::app_data::ContainerId;
+
#[derive(Debug, Clone)]
pub enum DockerMessage {
Update,
- Start(String),
- Restart(String),
- Pause(String),
- Unpause(String),
- Stop(String),
+ Start(ContainerId),
+ Restart(ContainerId),
+ Pause(ContainerId),
+ Unpause(ContainerId),
+ Stop(ContainerId),
Quit,
}
diff --git a/src/docker_data/mod.rs b/src/docker_data/mod.rs
index feb6314..d0a3e47 100644
--- a/src/docker_data/mod.rs
+++ b/src/docker_data/mod.rs
@@ -16,7 +16,7 @@ use tokio::{sync::mpsc::Receiver, task::JoinHandle};
use uuid::Uuid;
use crate::{
- app_data::{AppData, DockerControls},
+ app_data::{AppData, ContainerId, DockerControls},
app_error::AppError,
parse_args::CliArgs,
ui::GuiState,
@@ -24,16 +24,17 @@ use crate::{
mod message;
pub use message::DockerMessage;
-#[derive(Debug, Hash, Clone, PartialEq, Eq)]
+#[derive(Debug, Clone, Eq, Hash, PartialEq)]
enum SpawnId {
- Stats((String, Binate)),
- Log(String),
+ Stats((ContainerId, Binate)),
+ Log(ContainerId),
}
/// Cpu & Mem stats take twice as long as the update interval to get a value, so will have two being executed at the same time
/// SpawnId::Stats takes container_id and binate value to enable both cycles of the same container_id to be inserted into the hashmap
/// Binate value is toggled when all join handles have been spawned off
-#[derive(Debug, Hash, Clone, PartialEq, Eq, Copy)]
+/// Also effectively means that if the docker_update interval minimum will be 1000ms
+#[derive(Debug, Clone, Copy, Eq, Hash, PartialEq)]
enum Binate {
One,
Two,
@@ -61,7 +62,8 @@ pub struct DockerData {
}
impl DockerData {
- /// Use docker stats for work out current cpu usage
+ /// Use docker stats to caluclate current cpu usage
+ #[allow(clippy::cast_precision_loss)]
fn calculate_usage(stats: &Stats) -> f64 {
let mut cpu_percentage = 0.0;
let previous_cpu = stats.precpu_stats.cpu_usage.total_usage;
@@ -77,9 +79,8 @@ impl DockerData {
.cpu_stats
.cpu_usage
.percpu_usage
- .clone()
- .unwrap_or_default()
- .len() as u64
+ .as_ref()
+ .map_or(0, std::vec::Vec::len) as u64
}) as f64;
if system_delta > 0.0 && cpu_delta > 0.0 {
cpu_percentage = (cpu_delta / system_delta) * online_cpus * 100.0;
@@ -93,7 +94,7 @@ impl DockerData {
/// remove if from spawns hashmap when complete
async fn update_container_stat(
docker: Arc,
- id: String,
+ id: ContainerId,
app_data: Arc>,
is_running: bool,
spawns: Arc>>>,
@@ -101,7 +102,7 @@ impl DockerData {
) {
let mut stream = docker
.stats(
- &id,
+ id.get(),
Some(StatsOptions {
stream: false,
one_shot: !is_running,
@@ -113,22 +114,21 @@ impl DockerData {
let mem_stat = stats.memory_stats.usage.unwrap_or(0);
let mem_limit = stats.memory_stats.limit.unwrap_or(0);
- let some_key = stats
+ let op_key = stats
.networks
.as_ref()
.and_then(|networks| networks.keys().next().cloned());
let cpu_stats = Self::calculate_usage(&stats);
- let no_bytes = || (0, 0);
-
- let (rx, tx) = if let Some(key) = some_key {
- match stats.networks.unwrap_or_default().get(&key) {
- Some(data) => (data.rx_bytes, data.tx_bytes),
- None => no_bytes(),
- }
+ let (rx, tx) = if let Some(key) = op_key {
+ stats
+ .networks
+ .unwrap_or_default()
+ .get(&key)
+ .map_or((0, 0), |f| (f.rx_bytes, f.tx_bytes))
} else {
- no_bytes()
+ (0, 0)
};
if is_running {
@@ -150,15 +150,12 @@ impl DockerData {
}
/// Update all stats, spawn each container into own tokio::spawn thread
- async fn update_all_container_stats(&mut self, all_ids: &[(bool, String)]) {
- for (is_running, id) in all_ids.iter() {
+ fn update_all_container_stats(&mut self, all_ids: &[(bool, ContainerId)]) {
+ for (is_running, id) in all_ids {
let docker = Arc::clone(&self.docker);
let app_data = Arc::clone(&self.app_data);
let spawns = Arc::clone(&self.spawns);
- let id = id.clone();
-
let key = SpawnId::Stats((id.clone(), self.binate));
-
let spawn_key = key.clone();
self.spawns.lock().entry(key).or_insert_with(|| {
tokio::spawn(Self::update_container_stat(
@@ -177,7 +174,7 @@ impl DockerData {
/// Get all current containers, handle into ContainerItem in the app_data struct rather than here
/// Just make sure that items sent are guaranteed to have an id
/// Will ignore any container that contains `oxker` as an entry point
- pub async fn update_all_containers(&mut self) -> Vec<(bool, String)> {
+ pub async fn update_all_containers(&mut self) -> Vec<(bool, ContainerId)> {
let containers = self
.docker
.list_containers(Some(ListContainersOptions:: {
@@ -188,13 +185,13 @@ impl DockerData {
.unwrap_or_default();
let mut output = containers
- .iter()
+ .into_iter()
.filter_map(|f| match f.id {
Some(_) => {
if f.command.as_ref().map_or(false, |c| c.contains("oxker")) {
None
} else {
- Some(f.clone())
+ Some(f)
}
}
None => None,
@@ -208,13 +205,13 @@ impl DockerData {
// Just get the containers that are currently running, or being restarted, no point updating info on paused or dead containers
output
- .iter()
+ .into_iter()
.filter_map(|i| {
- i.id.as_ref().map(|id| {
+ i.id.map(|id| {
(
i.state == Some("running".to_owned())
|| i.state == Some("restarting".to_owned()),
- id.clone(),
+ ContainerId::from(id),
)
})
})
@@ -226,21 +223,20 @@ impl DockerData {
/// remove if from spawns hashmap when complete
async fn update_log(
docker: Arc,
- id: String,
+ id: ContainerId,
timestamps: bool,
- since: i64,
+ since: u64,
app_data: Arc>,
spawns: Arc>>>,
) {
let options = Some(LogsOptions:: {
stdout: true,
timestamps,
- since,
+ since: i64::try_from(since).unwrap_or_default(),
..Default::default()
});
- let mut logs = docker.logs(&id, options);
-
+ let mut logs = docker.logs(id.get(), options);
let mut output = vec![];
while let Some(value) = logs.next().await {
@@ -256,18 +252,21 @@ impl DockerData {
}
/// Update all logs, spawn each container into own tokio::spawn thread
- async fn init_all_logs(&mut self, all_ids: &[(bool, String)]) {
- for (_, id) in all_ids.iter() {
+ fn init_all_logs(&mut self, all_ids: &[(bool, ContainerId)]) {
+ for (_, id) in all_ids {
let docker = Arc::clone(&self.docker);
- let timestamps = self.timestamps;
- let id = id.clone();
let app_data = Arc::clone(&self.app_data);
let spawns = Arc::clone(&self.spawns);
let key = SpawnId::Log(id.clone());
self.spawns.lock().insert(
key,
tokio::spawn(Self::update_log(
- docker, id, timestamps, 0, app_data, spawns,
+ docker,
+ id.clone(),
+ self.timestamps,
+ 0,
+ app_data,
+ spawns,
)),
);
}
@@ -278,22 +277,26 @@ impl DockerData {
let all_ids = self.update_all_containers().await;
let optional_index = self.app_data.lock().get_selected_log_index();
if let Some(index) = optional_index {
- // this could be neater
- let id = self.app_data.lock().containers.items[index].id.clone();
- let key = SpawnId::Log(id.clone());
-
- self.spawns.lock().entry(key).or_insert_with(|| {
- let since = self.app_data.lock().containers.items[index].last_updated as i64;
- let docker = Arc::clone(&self.docker);
- let timestamps = self.timestamps;
- let app_data = Arc::clone(&self.app_data);
- let spawns = Arc::clone(&self.spawns);
- tokio::spawn(Self::update_log(
- docker, id, timestamps, since, app_data, spawns,
- ))
- });
+ if let Some(container) = self.app_data.lock().containers.items.get(index) {
+ self.spawns
+ .lock()
+ .entry(SpawnId::Log(container.id.clone()))
+ .or_insert_with(|| {
+ let docker = Arc::clone(&self.docker);
+ let app_data = Arc::clone(&self.app_data);
+ let spawns = Arc::clone(&self.spawns);
+ tokio::spawn(Self::update_log(
+ docker,
+ container.id.clone(),
+ self.timestamps,
+ container.last_updated,
+ app_data,
+ spawns,
+ ))
+ });
+ }
};
- self.update_all_container_stats(&all_ids).await;
+ self.update_all_container_stats(&all_ids);
}
/// Animate the loading icon
@@ -319,10 +322,10 @@ impl DockerData {
let loading_spin = self.loading_spin(loading_uuid).await;
let all_ids = self.update_all_containers().await;
- self.update_all_container_stats(&all_ids).await;
+ self.update_all_container_stats(&all_ids);
// Maybe only do a single one at first?
- self.init_all_logs(&all_ids).await;
+ self.init_all_logs(&all_ids);
if all_ids.is_empty() {
self.initialised = true;
@@ -346,7 +349,7 @@ impl DockerData {
match message {
DockerMessage::Pause(id) => {
let loading_spin = self.loading_spin(loading_uuid).await;
- if docker.pause_container(&id).await.is_err() {
+ if docker.pause_container(id.get()).await.is_err() {
app_data
.lock()
.set_error(AppError::DockerCommand(DockerControls::Pause));
@@ -355,7 +358,7 @@ impl DockerData {
}
DockerMessage::Restart(id) => {
let loading_spin = self.loading_spin(loading_uuid).await;
- if docker.restart_container(&id, None).await.is_err() {
+ if docker.restart_container(id.get(), None).await.is_err() {
app_data
.lock()
.set_error(AppError::DockerCommand(DockerControls::Restart));
@@ -365,7 +368,7 @@ impl DockerData {
DockerMessage::Start(id) => {
let loading_spin = self.loading_spin(loading_uuid).await;
if docker
- .start_container(&id, None::>)
+ .start_container(id.get(), None::>)
.await
.is_err()
{
@@ -377,7 +380,7 @@ impl DockerData {
}
DockerMessage::Stop(id) => {
let loading_spin = self.loading_spin(loading_uuid).await;
- if docker.stop_container(&id, None).await.is_err() {
+ if docker.stop_container(id.get(), None).await.is_err() {
app_data
.lock()
.set_error(AppError::DockerCommand(DockerControls::Stop));
@@ -386,13 +389,11 @@ impl DockerData {
}
DockerMessage::Unpause(id) => {
let loading_spin = self.loading_spin(loading_uuid).await;
- if docker.unpause_container(&id).await.is_err() {
+ if docker.unpause_container(id.get()).await.is_err() {
app_data
.lock()
.set_error(AppError::DockerCommand(DockerControls::Unpause));
};
- // loading sping take uuid to remove
- // stop_loading_sping(uuid)
self.stop_loading_spin(&loading_spin, loading_uuid);
self.update_everything().await;
}
diff --git a/src/input_handler/message.rs b/src/input_handler/message.rs
index cbefb49..f87e2e9 100644
--- a/src/input_handler/message.rs
+++ b/src/input_handler/message.rs
@@ -1,6 +1,6 @@
use crossterm::event::{KeyCode, MouseEvent};
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, Copy)]
pub enum InputMessages {
ButtonPress(KeyCode),
MouseEvent(MouseEvent),
diff --git a/src/input_handler/mod.rs b/src/input_handler/mod.rs
index 731b005..91f2418 100644
--- a/src/input_handler/mod.rs
+++ b/src/input_handler/mod.rs
@@ -77,8 +77,8 @@ impl InputHandler {
}
}
- /// Mouse button
- fn m_button(&mut self) {
+ /// Toggle the mouse capture (via input of the 'm' key)
+ fn m_key(&mut self) {
if self.mouse_capture {
match execute!(std::io::stdout(), DisableMouseCapture) {
Ok(_) => self
@@ -118,7 +118,7 @@ impl InputHandler {
/// Sort containers based on a given header, switch asc to desc if already sorted, else always desc
fn sort(&self, header: Header) {
- let mut output = Some((header.clone(), SortedOrder::Desc));
+ let mut output = Some((header, SortedOrder::Desc));
let mut locked_data = self.app_data.lock();
if let Some((h, order)) = locked_data.get_sorted().as_ref() {
if &SortedOrder::Desc == order && h == &header {
@@ -128,7 +128,7 @@ impl InputHandler {
locked_data.set_sorted(output);
}
- /// Send a quit message to docker, to abort all spawns, if error, quit here instead
+ /// Send a quit message to docker, to abort all spawns, if an error is return, set is_running to false here instead
async fn quit(&self) {
match self.docker_sender.send(DockerMessage::Quit).await {
Ok(_) => (),
@@ -137,6 +137,7 @@ impl InputHandler {
}
/// Handle any keyboard button events
+ #[allow(clippy::too_many_lines)]
async fn button_press(&mut self, key_code: KeyCode) {
let show_error = self.app_data.lock().show_error;
let show_info = self.gui_state.lock().show_help;
@@ -154,7 +155,7 @@ impl InputHandler {
match key_code {
KeyCode::Char('q' | 'Q') => self.quit().await,
KeyCode::Char('h' | 'H') => self.gui_state.lock().show_help = false,
- KeyCode::Char('m' | 'M') => self.m_button(),
+ KeyCode::Char('m' | 'M') => self.m_key(),
_ => (),
}
} else {
@@ -171,7 +172,7 @@ impl InputHandler {
KeyCode::Char('9') => self.sort(Header::Tx),
KeyCode::Char('q' | 'Q') => self.quit().await,
KeyCode::Char('h' | 'H') => self.gui_state.lock().show_help = true,
- KeyCode::Char('m' | 'M') => self.m_button(),
+ KeyCode::Char('m' | 'M') => self.m_key(),
KeyCode::Tab => {
// Skip control panel if no containers, could be refactored
let has_containers = self.app_data.lock().get_container_len() == 0;
diff --git a/src/main.rs b/src/main.rs
index f4b67b0..ae69b15 100644
--- a/src/main.rs
+++ b/src/main.rs
@@ -35,37 +35,32 @@ fn setup_tracing() {
async fn main() {
setup_tracing();
let args = CliArgs::new();
- let app_data = Arc::new(Mutex::new(AppData::default(args.clone())));
+ let app_data = Arc::new(Mutex::new(AppData::default(args)));
let gui_state = Arc::new(Mutex::new(GuiState::default()));
let is_running = Arc::new(AtomicBool::new(true));
- let docker_args = args.clone();
let docker_app_data = Arc::clone(&app_data);
let docker_gui_state = Arc::clone(&gui_state);
let (docker_sx, docker_rx) = tokio::sync::mpsc::channel(16);
// Create docker daemon handler, and only spawn up the docker data handler if ping returns non-error
-
match Docker::connect_with_socket_defaults() {
- Ok(docker) => {
- let docker = Arc::new(docker);
- match docker.ping().await {
- Ok(_) => {
- let docker = Arc::clone(&docker);
- let is_running = Arc::clone(&is_running);
- tokio::spawn(DockerData::init(
- docker_args,
- docker_app_data,
- docker,
- docker_gui_state,
- docker_rx,
- is_running,
- ));
- }
- Err(_) => app_data.lock().set_error(AppError::DockerConnect),
+ Ok(docker) => match docker.ping().await {
+ Ok(_) => {
+ let docker = Arc::new(docker);
+ let is_running = Arc::clone(&is_running);
+ tokio::spawn(DockerData::init(
+ args,
+ docker_app_data,
+ docker,
+ docker_gui_state,
+ docker_rx,
+ is_running,
+ ));
}
- }
+ Err(_) => app_data.lock().set_error(AppError::DockerConnect),
+ },
Err(_) => app_data.lock().set_error(AppError::DockerConnect),
}
let input_app_data = Arc::clone(&app_data);
@@ -85,7 +80,6 @@ async fn main() {
input_is_running,
));
- // Debug mode for testing, mostly pointless, doesn't take terminal nor draw gui
if args.gui {
let update_duration = std::time::Duration::from_millis(u64::from(args.docker_interval));
create_ui(
@@ -99,8 +93,9 @@ async fn main() {
.await
.unwrap_or(());
} else {
+ // Debug mode for testing, mostly pointless, doesn't take terminal nor draw gui
+ // TODO this needs to be improved to display something actually useful
loop {
- // TODO this needs to be improved to display something useful
info!("in debug mode");
tokio::time::sleep(std::time::Duration::from_millis(5000)).await;
}
diff --git a/src/parse_args/mod.rs b/src/parse_args/mod.rs
index a115aed..5c70ce9 100644
--- a/src/parse_args/mod.rs
+++ b/src/parse_args/mod.rs
@@ -3,29 +3,29 @@ use std::process;
use clap::Parser;
use tracing::error;
-#[derive(Parser, Debug, Clone)]
-#[clap(about, version, author)]
-
+#[derive(Parser, Debug, Clone, Copy)]
+#[allow(clippy::struct_excessive_bools)]
+#[command(version, about)]
pub struct CliArgs {
- /// Docker update interval in ms, minimum 1, reccomended 500+
+ /// Docker update interval in ms, minimum effectively 1000
#[clap(short = 'd', value_name = "ms", default_value_t = 1000)]
pub docker_interval: u32,
- /// Don't draw gui - for debugging - mostly pointless
- #[clap(short = 'g')]
- pub gui: bool,
-
/// Remove timestamps from Docker logs
#[clap(short = 't')]
pub timestamp: bool,
- /// Show raw logs, default is to remove ansi formatting
+ /// Attempt to colorize the logs, conflicts with "-r"
+ #[clap(short = 'c', conflicts_with = "raw")]
+ pub color: bool,
+
+ /// Show raw logs, default is to remove ansi formatting, conflicts with "-c"
#[clap(short = 'r', conflicts_with = "color")]
pub raw: bool,
- /// Attempt to colorize the logs
- #[clap(short = 'c', conflicts_with = "raw")]
- pub color: bool,
+ /// Don't draw gui - for debugging - mostly pointless
+ #[clap(short = 'g')]
+ pub gui: bool,
}
impl CliArgs {
diff --git a/src/ui/color_match.rs b/src/ui/color_match.rs
index a95dc88..8ad3112 100644
--- a/src/ui/color_match.rs
+++ b/src/ui/color_match.rs
@@ -7,14 +7,14 @@ pub mod log_sanitizer {
};
/// Attempt to colorize the given string to tui-rs standards
- pub fn colorize_logs(input: &str) -> Vec> {
+ pub fn colorize_logs<'a>(input: &str) -> Vec> {
vec![Spans::from(
categorise_text(input)
- .into_iter()
+ .iter()
.map(|i| {
- let fg_color = color_ansi_to_tui(i.fg.unwrap_or(CansiColor::White));
- let bg_color = color_ansi_to_tui(i.bg.unwrap_or(CansiColor::Black));
- let style = Style::default().bg(bg_color).fg(fg_color);
+ let style = Style::default()
+ .bg(color_ansi_to_tui(i.bg.unwrap_or(CansiColor::Black)))
+ .fg(color_ansi_to_tui(i.fg.unwrap_or(CansiColor::White)));
if i.blink.is_some() {
style.add_modifier(Modifier::SLOW_BLINK);
}
@@ -40,17 +40,17 @@ pub mod log_sanitizer {
}
/// Remove all ansi formatting from a given string and create tui-rs spans
- pub fn remove_ansi(input: &str) -> Vec> {
- let mut output = String::from("");
+ pub fn remove_ansi<'a>(input: &str) -> Vec> {
+ let mut output = String::new();
for i in categorise_text(input) {
output.push_str(i.text);
}
- raw(output)
+ raw(&output)
}
/// create tui-rs spans that exactly match the given strings
- pub fn raw(input: String) -> Vec> {
- vec![Spans::from(Span::raw(input))]
+ pub fn raw<'a>(input: &str) -> Vec> {
+ vec![Spans::from(Span::raw(input.to_owned()))]
}
/// Change from ansi to tui colors
diff --git a/src/ui/gui_state.rs b/src/ui/gui_state.rs
index 1239e20..f98b990 100644
--- a/src/ui/gui_state.rs
+++ b/src/ui/gui_state.rs
@@ -7,7 +7,7 @@ use uuid::Uuid;
use crate::app_data::Header;
-#[derive(Debug, PartialEq, std::hash::Hash, std::cmp::Eq, Clone, Copy)]
+#[derive(Debug, Clone, Copy, Eq, Hash, PartialEq)]
pub enum SelectablePanel {
Containers,
Commands,
@@ -38,6 +38,7 @@ impl SelectablePanel {
}
}
+#[derive(Debug, Copy, Clone)]
pub enum Region {
Panel(SelectablePanel),
Header(Header),
@@ -72,7 +73,7 @@ impl BoxLocation {
}
}
- // Should combine and just return a tuple?
+ // Should combine with get_vertical_constraints and just return a tuple of (vc, hc)?
pub const fn get_horizontal_constraints(
self,
blank_vertical: u16,
@@ -221,7 +222,7 @@ impl GuiState {
.filter(|i| i.1.intersects(rect))
.collect::>()
.get(0)
- .map(|data| data.0.clone())
+ .map(|data| *data.0)
}
/// Insert, or updates header area panel into heading_map
@@ -250,22 +251,22 @@ impl GuiState {
self.selected_panel = self.selected_panel.prev();
}
- /// Advance loading animation
+ /// Insert a new loading_uuid into hashset, and advance the animation by one frame
pub fn next_loading(&mut self, uuid: Uuid) {
self.loading_icon = self.loading_icon.next();
self.is_loading.insert(uuid);
}
- /// if is_loading, return loading animation frame, else single space
+ /// If is_loading has any entries, return the current loading_icon, else an emtpy string
pub fn get_loading(&mut self) -> String {
if self.is_loading.is_empty() {
- String::from(" ")
- } else {
+ String::new()
+ } else {
self.loading_icon.to_string()
}
}
- /// set is_loading to false, but keep animation frame at same state
+ /// Remove a loading_uuid from the is_loading hashset
pub fn remove_loading(&mut self, uuid: Uuid) {
self.is_loading.remove(&uuid);
}
diff --git a/src/ui/mod.rs b/src/ui/mod.rs
index 3f5fc07..b126dbd 100644
--- a/src/ui/mod.rs
+++ b/src/ui/mod.rs
@@ -93,7 +93,7 @@ async fn run_app(
break;
}
if terminal
- .draw(|f| draw_blocks::error(f, &AppError::DockerConnect, Some(seconds)))
+ .draw(|f| draw_blocks::error(f, AppError::DockerConnect, Some(seconds)))
.is_err()
{
return Err(AppError::Terminal);
@@ -107,6 +107,8 @@ async fn run_app(
if terminal.draw(|f| ui(f, &app_data, &gui_state)).is_err() {
return Err(AppError::Terminal);
}
+ // TODO could only draw if in gui mode, that way all inputs & docker commands will run, and can just trace!("{event"}) all over the place
+ // refactor this into own function, so can be called without drawing to the terminal
if crossterm::event::poll(input_poll_rate).unwrap_or(false) {
if let Ok(event) = event::read() {
if let Event::Key(key) = event {
@@ -146,7 +148,7 @@ fn ui(
) {
// set max height for container section, needs +4 to deal with docker commands list and borders
let height = app_data.lock().get_container_len();
- let height = if height < 12 { (height + 4) as u16 } else { 12 };
+ let height = if height < 12 { height + 4 } else { 12 };
let column_widths = app_data.lock().get_width();
let has_containers = !app_data.lock().containers.items.is_empty();
@@ -166,7 +168,13 @@ fn ui(
// Split into 3, containers+controls, logs, then graphs
let upper_main = Layout::default()
.direction(Direction::Vertical)
- .constraints([Constraint::Max(height as u16), Constraint::Percentage(50)].as_ref())
+ .constraints(
+ [
+ Constraint::Max(height.try_into().unwrap_or_default()),
+ Constraint::Percentage(50),
+ ]
+ .as_ref(),
+ )
.split(whole_layout[1]);
let top_split = if has_containers {
@@ -213,7 +221,7 @@ fn ui(
f,
has_containers,
&loading_icon,
- &sorted_by,
+ sorted_by,
gui_state,
);
@@ -233,6 +241,6 @@ fn ui(
if let Some(error) = has_error {
app_data.lock().show_error = true;
- draw_blocks::error(f, &error, None);
+ draw_blocks::error(f, error, None);
}
}
diff --git a/src/ui/draw_blocks.rs b/src/ui/draw_blocks.rs
index 1573a37..50c4177 100644
--- a/src/ui/draw_blocks.rs
+++ b/src/ui/draw_blocks.rs
@@ -39,6 +39,8 @@ const REPO: &str = env!("CARGO_PKG_REPOSITORY");
const DESCRIPTION: &str = env!("CARGO_PKG_DESCRIPTION");
const ORANGE: Color = Color::Rgb(255, 178, 36);
const MARGIN: &str = " ";
+const ARROW: &str = "▶ ";
+const CIRCLE: &str = "⚪ ";
/// Generate block, add a border if is the selected panel,
/// add custom title based on state of each panel
@@ -49,9 +51,6 @@ fn generate_block<'a>(
panel: SelectablePanel,
) -> Block<'a> {
gui_state.lock().update_map(Region::Panel(panel), area);
- let mut block = Block::default()
- .borders(Borders::ALL)
- .border_type(BorderType::Rounded);
let current_selected_panel = gui_state.lock().selected_panel;
let title = match panel {
SelectablePanel::Containers => {
@@ -64,9 +63,12 @@ fn generate_block<'a>(
SelectablePanel::Logs => {
format!(" {} {} ", panel.title(), app_data.lock().get_log_title())
}
- SelectablePanel::Commands => String::from(""),
+ SelectablePanel::Commands => String::new(),
};
- block = block.title(title);
+ let mut block = Block::default()
+ .borders(Borders::ALL)
+ .border_type(BorderType::Rounded)
+ .title(title);
if current_selected_panel == panel {
block = block.border_style(Style::default().fg(Color::LightCyan));
}
@@ -99,7 +101,7 @@ pub fn commands(
let items = List::new(items)
.block(block)
.highlight_style(Style::default().add_modifier(Modifier::BOLD))
- .highlight_symbol("▶ ");
+ .highlight_symbol(ARROW);
f.render_stateful_widget(
items,
@@ -107,10 +109,7 @@ pub fn commands(
&mut app_data.lock().containers.items[i].docker_controls.state,
);
} else {
- let debug_text = String::from("");
- let paragraph = Paragraph::new(debug_text)
- .block(block)
- .alignment(Alignment::Center);
+ let paragraph = Paragraph::new("").block(block).alignment(Alignment::Center);
f.render_widget(paragraph, area);
}
}
@@ -136,7 +135,7 @@ pub fn containers(
let mems = format!(
"{:>1} / {:>1}",
- i.mem_stats.back().unwrap_or(&ByteStats::new(0)),
+ i.mem_stats.back().unwrap_or(&ByteStats::default()),
i.mem_limit
);
@@ -153,7 +152,7 @@ pub fn containers(
format!(
"{}{:>width$}",
MARGIN,
- i.cpu_stats.back().unwrap_or(&CpuStats::new(0.0)),
+ i.cpu_stats.back().unwrap_or(&CpuStats::default()),
width = &widths.cpu.1
),
state_style,
@@ -166,7 +165,7 @@ pub fn containers(
format!(
"{}{:>width$}",
MARGIN,
- i.id.chars().take(8).collect::(),
+ i.id.get().chars().take(8).collect::(),
width = &widths.id.1
),
blue,
@@ -192,8 +191,7 @@ pub fn containers(
})
.collect::>();
if items.is_empty() {
- let debug_text = String::from("no containers running");
- let paragraph = Paragraph::new(debug_text)
+ let paragraph = Paragraph::new("no containers running")
.block(block)
.alignment(Alignment::Center);
f.render_widget(paragraph, area);
@@ -201,7 +199,7 @@ pub fn containers(
let items = List::new(items)
.block(block)
.highlight_style(Style::default().add_modifier(Modifier::BOLD))
- .highlight_symbol("⚪ ");
+ .highlight_symbol(CIRCLE);
f.render_stateful_widget(items, area, &mut app_data.lock().containers.state);
}
@@ -220,8 +218,7 @@ pub fn logs(
let init = app_data.lock().init;
if !init {
- let parsing_logs = format!("parsing logs {}", loading_icon);
- let paragraph = Paragraph::new(parsing_logs)
+ let paragraph = Paragraph::new(format!("parsing logs {}", loading_icon))
.style(Style::default())
.block(block)
.alignment(Alignment::Center);
@@ -237,7 +234,7 @@ pub fn logs(
let items = List::new(items)
.block(block)
- .highlight_symbol("▶ ")
+ .highlight_symbol(ARROW)
.highlight_style(Style::default().add_modifier(Modifier::BOLD));
f.render_stateful_widget(
items,
@@ -245,8 +242,7 @@ pub fn logs(
&mut app_data.lock().containers.items[index].logs.state,
);
} else {
- let debug_text = String::from("no logs found");
- let paragraph = Paragraph::new(debug_text)
+ let paragraph = Paragraph::new("no logs found")
.block(block)
.alignment(Alignment::Center);
f.render_widget(paragraph, area);
@@ -281,8 +277,8 @@ pub fn chart(
.data(&mem.0)];
let cpu_stats = CpuStats::new(cpu.0.last().map_or(0.00, |f| f.1));
let mem_stats = ByteStats::new(mem.0.last().map_or(0, |f| f.1 as u64));
- let cpu_chart = make_chart(&cpu.2, "cpu", cpu_dataset, &cpu_stats, &cpu.1);
- let mem_chart = make_chart(&mem.2, "memory", mem_dataset, &mem_stats, &mem.1);
+ let cpu_chart = make_chart(cpu.2, "cpu", cpu_dataset, &cpu_stats, &cpu.1);
+ let mem_chart = make_chart(mem.2, "memory", mem_dataset, &mem_stats, &mem.1);
f.render_widget(cpu_chart, area[0]);
f.render_widget(mem_chart, area[1]);
@@ -292,7 +288,7 @@ pub fn chart(
/// Create charts
fn make_chart<'a, T: Stats + Display>(
- state: &State,
+ state: State,
name: &'a str,
dataset: Vec>,
current: &'a T,
@@ -340,14 +336,15 @@ fn make_chart<'a, T: Stats + Display>(
)
}
-/// Draw heading bar at top of program, always visible
+/// Draw heading bar at top of program, always visible
+#[allow(clippy::too_many_lines)]
pub fn heading_bar(
area: Rect,
columns: &Columns,
f: &mut Frame<'_, B>,
has_containers: bool,
loading_icon: &str,
- sorted_by: &Option<(Header, SortedOrder)>,
+ sorted_by: Option<(Header, SortedOrder)>,
gui_state: &Arc>,
) {
let block = || Block::default().style(Style::default().bg(Color::Magenta).fg(Color::Black));
@@ -405,14 +402,14 @@ pub fn heading_bar(
width = width - block.2
),
};
- let count = text.chars().count() as u16;
+ let count = u16::try_from(text.chars().count()).unwrap_or_default();
let status = Paragraph::new(text)
.block(block.0)
.alignment(Alignment::Left);
(status, count)
};
- // Meta data for iterate over to create blocks and correct widths
+ // Meta data to iterate over to create blocks with correct widths
let header_meta = [
(Header::State, columns.state.1),
(Header::Status, columns.status.1),
@@ -429,18 +426,21 @@ pub fn heading_bar(
.iter()
.map(|i| {
let header_block = gen_header(&i.0, i.1);
- (header_block.0, i.0.clone(), Constraint::Max(header_block.1))
+ (header_block.0, i.0, Constraint::Max(header_block.1))
})
.collect::>();
let suffix = if info_visible { "exit" } else { "show" };
let info_text = format!("( h ) {} help {}", suffix, MARGIN);
- let info_width = info_text.chars().count() as u16;
+ let info_width = info_text.chars().count();
- let column_width = area.width - info_width;
+ let column_width = usize::from(area.width) - info_width;
let column_width = if column_width > 0 { column_width } else { 1 };
let splits = if has_containers {
- vec![Constraint::Min(column_width), Constraint::Min(info_width)]
+ vec![
+ Constraint::Min(column_width.try_into().unwrap_or_default()),
+ Constraint::Min(info_width.try_into().unwrap_or_default()),
+ ]
} else {
vec![Constraint::Percentage(100)]
};
@@ -469,11 +469,12 @@ pub fn heading_bar(
.block(block())
.alignment(Alignment::Right);
+ // If no containers, don't display the headers, could maybe do this first?
let index = if has_containers { 1 } else { 0 };
f.render_widget(paragraph, split_bar[index]);
}
/// From a given &String, return the maximum number of chars on a single line
fn max_line_width(text: &str) -> usize {
let mut max_line_width = 0;
text.lines().into_iter().for_each(|line| {
@@ -486,6 +487,7 @@ fn max_line_width(text: &str) -> usize {
}
/// Draw the help box in the centre of the screen
+/// TODO this is message, should make every line it's own renderable span
pub fn help_box(f: &mut Frame<'_, B>) {
let title = format!(" {} ", VERSION);
@@ -536,20 +538,21 @@ pub fn help_box(f: &mut Frame<'_, B>) {
.border_type(BorderType::Rounded)
.border_style(Style::default().fg(Color::Black));
- let area = popup(
- lines as u16,
- max_line_width as u16,
- f.size(),
- BoxLocation::MiddleCentre,
- );
+ let area = popup(lines, max_line_width, f.size(), BoxLocation::MiddleCentre);
let split_popup = Layout::default()
.direction(Direction::Vertical)
.constraints(
[
- Constraint::Max(NAME_TEXT.lines().count() as u16),
- Constraint::Max(description_text.lines().count() as u16),
- Constraint::Max(help_text.lines().count() as u16),
+ Constraint::Max(NAME_TEXT.lines().count().try_into().unwrap_or_default()),
+ Constraint::Max(
+ description_text
+ .lines()
+ .count()
+ .try_into()
+ .unwrap_or_default(),
+ ),
+ Constraint::Max(help_text.lines().count().try_into().unwrap_or_default()),
]
.as_ref(),
)
@@ -564,7 +567,7 @@ pub fn help_box(f: &mut Frame<'_, B>) {
}
/// Draw an error popup over whole screen
-pub fn error(f: &mut Frame<'_, B>, error: &AppError, seconds: Option) {
+pub fn error(f: &mut Frame<'_, B>, error: AppError, seconds: Option) {
let block = Block::default()
.title(" Error ")
.border_type(BorderType::Rounded)
@@ -600,12 +603,7 @@ pub fn error(f: &mut Frame<'_, B>, error: &AppError, seconds: Option
.block(block)
.alignment(Alignment::Center);
- let area = popup(
- lines as u16,
- max_line_width as u16,
- f.size(),
- BoxLocation::MiddleCentre,
- );
+ let area = popup(lines, max_line_width, f.size(), BoxLocation::MiddleCentre);
f.render_widget(Clear, area);
f.render_widget(paragraph, area);
}
@@ -629,32 +627,33 @@ pub fn info(f: &mut Frame<'_, B>, text: String) {
.block(block)
.alignment(Alignment::Center);
- let area = popup(
- lines as u16,
- max_line_width as u16,
- f.size(),
- BoxLocation::BottomRight,
- );
+ let area = popup(lines, max_line_width, f.size(), BoxLocation::BottomRight);
f.render_widget(Clear, area);
f.render_widget(paragraph, area);
}
/// draw a box in the one of the BoxLocations, based on max line width + number of lines
-fn popup(text_lines: u16, text_width: u16, r: Rect, box_location: BoxLocation) -> Rect {
+fn popup(text_lines: usize, text_width: usize, r: Rect, box_location: BoxLocation) -> Rect {
// Make sure blank_space can't be an negative, as will crash
- let blank_vertical = if r.height > text_lines {
- (r.height - text_lines) / 2
+ let blank_vertical = if usize::from(r.height) > text_lines {
+ (usize::from(r.height) - text_lines) / 2
} else {
1
};
- let blank_horizontal = if r.width > text_width {
- (r.width - text_width) / 2
+ let blank_horizontal = if usize::from(r.width) > text_width {
+ (usize::from(r.width) - text_width) / 2
} else {
1
};
- let v_constraints = box_location.get_vertical_constraints(blank_vertical, text_lines);
- let h_constraints = box_location.get_horizontal_constraints(blank_horizontal, text_width);
+ let v_constraints = box_location.get_vertical_constraints(
+ blank_vertical.try_into().unwrap_or_default(),
+ text_lines.try_into().unwrap_or_default(),
+ );
+ let h_constraints = box_location.get_horizontal_constraints(
+ blank_horizontal.try_into().unwrap_or_default(),
+ text_width.try_into().unwrap_or_default(),
+ );
let indexes = box_location.get_indexes();
@@ -469,11 +469,12 @@ pub fn heading_bar(
f.render_widget(paragraph, split_bar[index]);
}
-/// From a given &String, return the maximum number of chars on a single line
+/// From a given &str, return the maximum number of chars on a single line
fn max_line_width(text: &str) -> usize {
let mut max_line_width = 0;